@@ -94,6 +94,9 @@ class Cling(object):
             return self.method_not_allowed(environ, start_response)
         path_info = environ.get('PATH_INFO', '')
         full_path = self._full_path(path_info)
+        # guard against arbitrary file retrieval
+        if not path.abspath(full_path).startswith(path.abspath(self.root)):
+            return self.not_found(environ, start_reponse)
         if path.isdir(full_path):
             if full_path[-1] <> '/' or full_path == self.root:
                 location = util.request_uri(environ, include_query=False) + '/'
